package opts

import (
	"context"

	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"google.golang.org/grpc"

	"gitea.libretechconsulting.com/rmcguire/go-app/pkg/config"
)

type GRPCOpts struct {
	*config.GRPCConfig // Settings configurable by env or yaml
	*AppGRPC           // Settings provided in code
}

type AppGRPC struct {
	Services           []*GRPCService
	UnaryInterceptors  []grpc.UnaryServerInterceptor
	StreamInterceptors []grpc.StreamServerInterceptor
	GRPCOpts           []grpc.ServerOption
	GRPCDialOpts       []grpc.DialOption        // Map ServerOptions to DialOpts for GRPC Gateway support
	GRPCGatewayOpts    []runtime.ServeMuxOption // Configure grpc-gateway ServeMux
	GRPCDone           <-chan error
	gatewayMux         *runtime.ServeMux
}

type GRPCService struct {
	Name                string               // Descriptive name of service
	Type                *grpc.ServiceDesc    // Type (from protoc generated code)
	Service             any                  // Implementation of GRPCService.Type (ptr)
	GwRegistrationFuncs []GwRegistrationFunc // Gateway regustration handler funcs
}

type GwRegistrationFunc func(context.Context, *runtime.ServeMux, *grpc.ClientConn) error

func (a *AppGRPC) SetGatewayMux(mux *runtime.ServeMux) {
	a.gatewayMux = mux
}

func (a *AppGRPC) GetGatewayMux() *runtime.ServeMux {
	return a.gatewayMux
}
